#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include "portaudio.h"
#include "ring_buffer.h"
#include "output_stream.h"
 

PaStream *output_stream;
DualRingBuffer *output_dual_buffer;

static int output_stream_callback( const void *inputBuffer, void *outputBuffer,
                            unsigned long framesPerBuffer,
                            const PaStreamCallbackTimeInfo* timeInfo,
                            PaStreamCallbackFlags statusFlags,
                            void *userData )
{
    RingBuffer *buffer_1, *buffer_2;
    DualRingBuffer *dual_buffer = (DualRingBuffer*)userData;
    float *out = (float*)outputBuffer;
    unsigned long i;
    float *ch1;
    float *ch2;
    float x;
    int err1, err2;
    buffer_1 = dual_buffer->buffer_1;
    buffer_2 = dual_buffer->buffer_2;
    (void) timeInfo; /* Prevent unused variable warnings. */
    (void) statusFlags;
    (void) inputBuffer;
    err1 = err2 = 0;
    if (buffer_1!=0) err1 = ring_buffer_read(buffer_1,&ch1);
    if (buffer_2!=0) err2 = ring_buffer_read(buffer_2,&ch2);
    //printf("%d %d\n",err1,err2);
    if ((err1>0)||(err2>0)) {
           for( i=0; i<framesPerBuffer; i++ ) {
           *out++ = 0.0;
           *out++ = 0.0;
        }  
    }
    else if (buffer_1==NULL) {
         for( i=0; i<framesPerBuffer; i++ ) {
           x = ch2[i];
           *out++ = x;
           *out++ = x;
        } 
    }
    else if (buffer_2==NULL) {
         for( i=0; i<framesPerBuffer; i++ ) {
           x = ch1[i];
           *out++ = x;
           *out++ = x;
        } 
    }
    else {
        for( i=0; i<framesPerBuffer; i++ ) {
           *out++ = ch1[i];
           *out++ = ch2[i];
        }
    }
    return paContinue;
}

static void output_stream_finished( void* userData )
{
       
}

int output_stream_start(int sample_rate, RingBuffer *buffer_1, RingBuffer *buffer_2) {
    PaStreamParameters outputParameters;
    PaError err;
    
    output_dual_buffer = (DualRingBuffer*) malloc(sizeof(DualRingBuffer));
    output_dual_buffer->buffer_1 = buffer_1;
    output_dual_buffer->buffer_2 = buffer_2;
    
    err = Pa_Initialize();
    if( err != paNoError ) {Pa_Terminate(); fprintf( stderr, "Error : %s\n", Pa_GetErrorText( err ) ); return err;}
    outputParameters.device = Pa_GetDefaultOutputDevice(); /* default output device */
    if (outputParameters.device == paNoDevice) {
      fprintf(stderr,"Error: No default output device.\n");
      Pa_Terminate();
      return 1;
    }
    outputParameters.channelCount = 2;       /* stereo output */
    outputParameters.sampleFormat = paFloat32; /* 32 bit floating point output */
    outputParameters.suggestedLatency = Pa_GetDeviceInfo( outputParameters.device )->defaultLowOutputLatency;
    outputParameters.hostApiSpecificStreamInfo = NULL;
    err = Pa_OpenStream(
              &output_stream,
              NULL, /* no input */
              &outputParameters,
              sample_rate,
              buffer_1->samples_per_block,
              0, // clip ON
              output_stream_callback,
              output_dual_buffer );
    if( err != paNoError ) {Pa_Terminate(); fprintf( stderr, "Error : %s\n", Pa_GetErrorText( err ) ); return err;}
    err = Pa_SetStreamFinishedCallback( output_stream, &output_stream_finished);
    if( err != paNoError ) {Pa_Terminate(); fprintf( stderr, "Error : %s\n", Pa_GetErrorText( err ) ); return err;}
    err = Pa_StartStream( output_stream );
    if( err != paNoError ) {Pa_Terminate(); fprintf( stderr, "Error : %s\n", Pa_GetErrorText( err ) ); return err;}
    
    return -1;
    
}

int output_stream_stop(int seconds) {
    PaError err;
    Pa_Sleep( seconds * 1000 );
    err = Pa_StopStream( output_stream );
    if( err != paNoError ) {Pa_Terminate(); fprintf( stderr, "Error : %s\n", Pa_GetErrorText( err ) ); return err;}
    err = Pa_CloseStream( output_stream );
    if( err != paNoError ) {Pa_Terminate(); fprintf( stderr, "Error : %s\n", Pa_GetErrorText( err ) ); return err;}
    Pa_Terminate();
    free(output_dual_buffer);
    return -1;
}

