#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ring_buffer.h"


int ring_buffer_init(RingBuffer *buffer, unsigned long number_blocks_exponent, unsigned long samples_per_block) {
    unsigned long size; 
    int iw;
    buffer->samples_per_block = samples_per_block;
    buffer->number_blocks = 1 << number_blocks_exponent;
    buffer->index_mask = buffer->number_blocks-1;
    buffer->block_write_index = 0;
    buffer->block_read_index = 0;
    buffer->num_blocks_available = 0;
    buffer->block_size = buffer->samples_per_block ;
    size =   buffer->number_blocks * buffer->block_size*sizeof(float);
    buffer->data = (float*) malloc(size);
    return -1;
}

int ring_buffer_delete(RingBuffer *buffer) {
    free(buffer->data);
    return -1;
}

int ring_buffer_write(RingBuffer *buffer, float *data) {
    unsigned long float_write_index;
    unsigned long size;
    float_write_index = buffer->block_write_index * buffer->block_size;
    size = buffer->block_size*sizeof(float);
    memcpy(buffer->data+float_write_index,data,size);
    buffer->block_write_index = (buffer->block_write_index + 1) & buffer->index_mask;
    buffer->num_blocks_available++;
    if  (buffer->num_blocks_available == buffer->number_blocks) {
        printf("full buffer %d : %d\n",buffer->id, buffer->block_write_index);  
    }
    //printf("buffer %d write block %d\n",buffer->id,buffer->block_write_index);
    return -1;
}

int ring_buffer_write_sample(RingBuffer *buffer, float data, int index) {
    unsigned long float_write_index;
    float_write_index = buffer->block_write_index * buffer->block_size + index;
    buffer->data[float_write_index] = data;
    //printf("buffer %d write sample %d\n",buffer->id,float_write_index);
    
    return -1;
}



int ring_buffer_write_next_block(RingBuffer *buffer) {
    buffer->block_write_index = (buffer->block_write_index + 1) & buffer->index_mask;
    buffer->num_blocks_available++;
    if  (buffer->num_blocks_available == buffer->number_blocks) {
        printf("full buffer %d : %d\n", buffer->id, buffer->block_write_index);  
    }
    return -1;   
}

int ring_buffer_read(RingBuffer *buffer, float **data) {
    unsigned long float_read_index;
    //printf("read : %lu\n",buffer->num_blocks_available);
    if (buffer->num_blocks_available==0) return RING_BUFFER_NOREAD_AVAILABLE;
    float_read_index = buffer->block_read_index * buffer->block_size;
    *data = buffer->data + float_read_index;
    buffer->block_read_index = (buffer->block_read_index + 1) & buffer->index_mask;
    buffer->num_blocks_available--;
    return -1;
}

int ring_buffer_full(RingBuffer *buffer) {
    if  (buffer->num_blocks_available >= buffer->number_blocks) return 1;
    return 0;
}

int ring_buffer_empty(RingBuffer *buffer) {
    if (buffer->num_blocks_available == 0) return 1;
    return 0;
       
}

int ring_buffer_available(RingBuffer *buffer, int *available) {
       *available = buffer->num_blocks_available;
       return -1;
}


