#define SP5 1
#define PCI 2
#define MSGSIZE 200
#define ECHANT_MEM 1
#define ECHANT_PERM 2
#define NB_TAMPONS 0x10
#define MASQUE_INDICE_TAMPONS 0FF
#define FILTRE_W_TAILLE 0x10


typedef struct {
    unsigned char errormsg[MSGSIZE];
    int (*can_ouvrir)(void*);
    int (*can_fermer)(void*);
    int (*can_reset)(void*);
    int (*can_config_entrees)(void*, int *listeVoies, int nbVoies, double *listeCalibres, int nbCalibes, int *listeDiff, int nbDiff);
    int (*can_config_trigger)(void*, int externe, int voie, double seuil, int montant, int pretrigger, int pretriggerSouple, int hysteresis);
    int (*can_config_echantillon)(void*, double temps,  int nbpoints);
    int (*can_config_echantillon_permanent)(void*, double temps,  int nbpoints);
    int (*can_acquerir)(void*);
    int (*can_acquerir_permanent)(void*);
    int (*can_acquerir_permanent_sansfin)(void*);
    int (*can_stopper_acquisition)(void*);
    int (*can_acquerir_avec_sorties)(void*,double *valeurs1, int nbvaleurs1,double *valeurs2, int nbvaleurs2); 
    int (*can_config_sortie)(void*,int nsortie, double te, double *valeurs, int nbvaleurs, int repetition);
    int (*can_declencher_sorties)(void*,int ns1, int ns2);
    int (*can_stopper_sorties)(void*,int ns1, int ns2);
    int (*can_activer_lecture)(void*,int* listeVoies, int nbVoie, int latch);
    int (*can_lire)(void*);
    int (*can_desactiver_lecture)(void *);
    int (*can_ecrire)(void*,int ns1, double value1, int ns2, double value2);
    int (*can_lancer)(void*);
    int (*can_lancer_permanent)(void*);
    int (*can_lancer_permanent_sansfin)(void*);
    int (*can_lancer_avec_sorties)(void*,double *valeurs1, int nbvaleurs1,double *valeurs2, int nbvaleurs2); 
    int (*can_portC_config)(void*,unsigned char bit, unsigned char etat);
    int (*can_portC_ecrire)(void*,unsigned char bit, unsigned char etat);
    int (*can_portC_lire)(void*,unsigned char bit);
    int (*can_portB_config)(void*,unsigned char bit, unsigned char etat);
    int (*can_portB_ecrire)(void*,unsigned char bit, unsigned char etat);
    int (*can_portB_lire)(void*,unsigned char bit);
    int (*can_config_filtre)(void*,double *A, int nA, double *B, int nB);
    int (*can_config_compteur)(void*,int entree, int front_debut, int front_fin, double hysteresis, double duree);
    int (*can_compteur)(void*);
    int (*can_lire_compteur)(void*);
    int (*can_config_chrono)(void*,int entree, int front_debut, int front_fin, double hysteresis);
    int(*can_chrono)(void*);
    int(*can_lire_chrono)(void*);
    int(*can_afficher_calibrage)(void*);
    
    unsigned char present;
    char type;
    short voiesEA[8]; // voies d'entrée sélectionnées
    int nombreEA; // nombre de voies d'entrée 
    int calibreEA[8]; // calibres des voies de 0 à 7
    char nbcalibres; // nombre de calibres du CAN
    double maxCalibreEA[7]; // tensions maximales des calibres
    double ampli_voies[7];
    double offset_voies[7];
    double temin;
    double tempsEchantillon;
    char type_echant;
    double tdecalEA[8];//décalage temporel des voies d'entrée
    int nbpointsEAdemande;
    int nbpointsEA;
    int nbpointsEAlus;
    int indiceCirc;
    int acquisition;
    double *tabEntrees;
    double *tabTemps;
    double *tabEntreesFiltrees;
    short masqueQ;
    
    int pos_tampon_ecriture;
    int pos_tampon_lecture;
    int nb_tampons_dispo;
    
    int nbpointsFiltres;
    char filtrage;
    int n_filtreA;
    double *filtreA;
    int n_filtreB;
    double *filtreB;
    
    int nbpointsS1, nbpointsS2;
    double *valeursS1, *valeursS2;
    short *tabSortieS1, *tabSortieS2;
    int sortieSelect[2];
    double tempsEchantillonSorties;
    int repetition;
    int voiesEAdirect[8];
    int nombreEAdirect;
    double tensionsLues[8];
    int tailleFifo;
    int ramAdc, ramDac0, ramDac1;
    unsigned char bitC;
    
    unsigned long int timer_low;
    unsigned long int timer_high;
    
    
} Handle;

int sp5_ouvrir(Handle *handle);
int sp5_fermer(Handle *handle);
int sp5_reset(Handle *handle);
int sp5_config_entrees(Handle *handle,int *listeVoies, int nbVoies, double *listeCalibres, int nbCalibres, int *listeDiff, int nbDiff);
int sp5_config_trigger(Handle *handle,int externe, int voie, double seuil, int montant, int pretrigger, int pretriggerSouple, int hysteresis);
int sp5_config_echantillon(Handle *handle, double temps,  int nbpoints);
int sp5_config_echantillon_permanent(Handle *handle, double temps,  int nbpoints);
int sp5_acquerir(Handle *handle);
int sp5_acquerir_permanent(Handle *handle);
int sp5_acquerir_permanent_sansfin(Handle *handle);
int sp5_stopper_acquisition(Handle *handle);
int sp5_acquerir_avec_sorties(Handle *handle, double *valeurs1, int nbvaleurs1,double *valeurs2, int nbvaleurs2);
int sp5_config_sortie(Handle *handle, int nsortie, double te, double *valeurs, int nbvaleurs, int repetition);
int sp5_declencher_sorties(Handle *handle, int ns1, int ns2);
int sp5_stopper_sorties(Handle *handle, int ns1, int ns2);
int sp5_activer_lecture(Handle *handle, int* listeVoies, int nbVoies, int latch);
int sp5_lire(Handle *handle);
int sp5_desactiver_lecture(Handle *handle);
int sp5_ecrire(Handle *handle, int ns1, double value1, int ns2, double value2);
int sp5_lancer(Handle *handle);
int sp5_lancer_permanent(Handle *handle);
int sp5_lancer_permanent_sansfin(Handle *handle);
int sp5_lancer_avec_sorties(Handle *handle,double *valeurs1, int nbvaleurs1,double *valeurs2, int nbvaleurs2);
int sp5_portC_config(Handle *handle, unsigned char bit, unsigned char etat);
int sp5_portC_ecrire(Handle *handle, unsigned char bit, unsigned char etat);
int sp5_portC_lire(Handle *handle, unsigned char bit);
int sp5_portB_config(Handle *handle, unsigned char bit, unsigned char etat);
int sp5_portB_ecrire(Handle *handle, unsigned char bit, unsigned char etat);
int sp5_portB_lire(Handle *handle, unsigned char bit);
int sp5_config_filtre(Handle *handle, double *A, int nA, double *B, int nB);
int sp5_config_compteur(Handle *handle,int entree, int front_debut, int front_fin, double hysteresis, double duree);
int sp5_compteur(Handle *handle);
int sp5_lire_compteur(Handle *handle);
int sp5_config_chrono(Handle *handle,int entree, int front_debut, int front_fin, double hysteresis);
int sp5_chrono(Handle *handle);
int sp5_lire_chrono(Handle *handle);
int sp5_afficher_calibrage(Handle *handle);

int pci_ouvrir(Handle *handle);
int pci_fermer(Handle *handle);
int pci_config_entrees(Handle *handle,int *listeVoies, int nbVoies, double *listeCalibres, int nbCalibres, int *listeDiff, int nbDiff);
int pci_config_trigger(Handle *handle,int externe, int voie, double seuil, int montant, int pretrigger, int pretriggerSouple, int hysteresis);
int pci_config_echantillon(Handle *handle, double temps,  int nbpoints);
int pci_acquerir(Handle *handle);
int pci_acquerir_avec_sorties(Handle *handle, double *valeurs1, int nbvaleurs1,double *valeurs2, int nbvaleurs2);
int pci_config_sortie(Handle *handle, int nsortie, double te, double *valeurs, int nbvaleurs, int repetition);
int pci_declencher_sorties(Handle *handle, int ns1, int ns2);
int pci_stopper_sorties(Handle *handle, int ns1, int ns2);
int pci_activer_lecture(Handle *handle, int* listeVoies, int nbVoies, int latch);
int pci_lire(Handle *handle);
int pci_desactiver_lecture(Handle *handle);
int pci_ecrire(Handle *handle, int ns1, double value1, int ns2, double value2);
int pci_lancer(Handle *handle);
int pci_lancer_avec_sorties(Handle *handle,double *valeurs1, int nbvaleurs1,double *valeurs2, int nbvaleurs2);
